import { commonsWaitEngineConfigure, commonsWaitFrame } from '../../../../WebPd_compiler/dist/src/stdlib/commons.js';
import { renderIf } from '../../../../WebPd_compiler/dist/src/functional-helpers.js';
import { assertOptionalString } from '../validation.js';
import { EMPTY_BUS_NAME, build, stateVariables, messageSetSendReceive, declareControlSendReceive } from './controls-base.js';
import { messageBuses } from '../global-code/buses.js';
import { bangUtils } from '../global-code/core.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
// ------------------------------- node builder ------------------------------ //
const builder = {
    translateArgs: ({ args: [init, receive, send] }) => ({
        outputOnLoad: !!init,
        sendBusName: assertOptionalString(send) || EMPTY_BUS_NAME,
        receiveBusName: assertOptionalString(receive) || EMPTY_BUS_NAME,
    }),
    build,
};
// ------------------------------- generateDeclarations ------------------------------ //
const generateDeclarations = (context) => {
    const { state, snds, node: { id, args }, macros: { Var, Func }, compilation: { codeVariableNames: { nodes } } } = context;
    return `
        function ${state.funcMessageReceiver} ${Func([
        Var('m', 'Message'),
    ], 'void')} {
            ${messageSetSendReceive(context)}
            else {
                const ${Var('outMessage', 'Message')} = msg_bang()
                ${nodes[id].snds.$0}(outMessage)
                if (${state.sendBusName} !== "${EMPTY_BUS_NAME}") {
                    msgBusPublish(${state.sendBusName}, outMessage)
                }
                return
            }
        }

        ${declareControlSendReceive(context)}

        ${renderIf(args.outputOnLoad, `commons_waitFrame(0, () => ${snds.$0}(msg_bang()))`)}
    `;
};
// ------------------------------- generateMessageReceivers ------------------------------ //
const generateMessageReceivers = (context) => {
    const { state, globs } = context;
    return ({
        '0': `
            ${state.funcMessageReceiver}(${globs.m})
            return
        `,
    });
};
// ------------------------------------------------------------------- //
const nodeImplementation = {
    generateDeclarations,
    generateMessageReceivers,
    stateVariables,
    dependencies: [
        bangUtils,
        messageBuses,
        commonsWaitEngineConfigure,
        commonsWaitFrame,
    ],
};

export { builder, nodeImplementation };
