import { assertOptionalNumber } from '../validation.js';
import { coldFloatInlet } from '../standard-message-receivers.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
const stateVariables = {
    threshold: 1
};
// ------------------------------- node builder ------------------------------ //
const builder = {
    translateArgs: ({ args }) => ({
        threshold: assertOptionalNumber(args[0]) || 0
    }),
    build: () => ({
        inlets: {
            '0': { type: 'message', id: '0' },
            '1': { type: 'message', id: '1' },
        },
        outlets: {
            '0': { type: 'message', id: '0' },
            '1': { type: 'message', id: '1' },
        },
    }),
};
// ------------------------------- generateDeclarations ------------------------------ //
const generateDeclarations = ({ node, state, macros: { Var }, }) => `
    let ${Var(state.threshold, 'Float')} = ${node.args.threshold}
`;
// ------------------------------- generateMessageReceivers ------------------------------ //
const generateMessageReceivers = ({ snds, globs, state, macros: { Var } }) => ({
    '0': `
    if (msg_isMatching(${globs.m}, [MSG_FLOAT_TOKEN])) {
        const ${Var('value', 'Float')} = msg_readFloatToken(${globs.m}, 0)
        if (value >= ${state.threshold}) {
            ${snds[1]}(msg_floats([value]))
        } else {
            ${snds[0]}(msg_floats([value]))
        }
        return
    }
    `,
    '1': coldFloatInlet(globs.m, state.threshold),
});
// ------------------------------------------------------------------- //
const nodeImplementation = { generateMessageReceivers, stateVariables, generateDeclarations };

export { builder, nodeImplementation };
