import { commonsWaitEngineConfigure } from '../../../../WebPd_compiler/dist/src/stdlib/commons.js';
import { assertOptionalNumber } from '../validation.js';
import { coldFloatInletWithSetter } from '../standard-message-receivers.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
const stateVariables = {
    phase: 1,
    J: 1,
    funcSetPhase: 1,
};
// ------------------------------- node builder ------------------------------ //
const builder = {
    translateArgs: (pdNode) => ({
        frequency: assertOptionalNumber(pdNode.args[0]) || 0,
    }),
    build: () => ({
        inlets: {
            '0': { type: 'signal', id: '0' },
            '1': { type: 'message', id: '1' },
        },
        outlets: {
            '0': { type: 'signal', id: '0' },
        },
    }),
    configureMessageToSignalConnection: (inletId, { frequency }) => {
        if (inletId === '0') {
            return { initialSignalValue: frequency };
        }
        return undefined;
    },
};
const makeNodeImplementation = ({ coeff, generateOperation, }) => {
    // ------------------------------ generateDeclarations ------------------------------ //
    const generateDeclarations = ({ state, globs, macros: { Var, Func }, }) => `
        let ${Var(state.phase, 'Float')} = 0
        let ${Var(state.J, 'Float')}

        function ${state.funcSetPhase} ${Func([
        Var('phase', 'Float')
    ], 'void')} { ${state.phase} = phase % 1.0${coeff ? ` * ${coeff}` : ''} }

        commons_waitEngineConfigure(() => {
            ${state.J} = ${coeff ? `${coeff}` : '1'} / ${globs.sampleRate}
        })
    `;
    // ------------------------------- generateLoop ------------------------------ //
    const generateLoop = ({ ins, state, outs }) => `
        ${outs.$0} = ${generateOperation(state.phase)}
        ${state.phase} += (${state.J} * ${ins.$0})
    `;
    // ------------------------------- generateMessageReceivers ------------------------------ //
    const generateMessageReceivers = ({ globs, state }) => ({
        '1': coldFloatInletWithSetter(globs.m, state.funcSetPhase),
    });
    return {
        generateDeclarations,
        generateMessageReceivers,
        generateLoop,
        stateVariables,
        dependencies: [commonsWaitEngineConfigure]
    };
};
// ------------------------------------------------------------------- //
const nodeImplementations = {
    'osc~': makeNodeImplementation({
        coeff: '2 * Math.PI',
        generateOperation: (phase) => `Math.cos(${phase})`
    }),
    'phasor~': makeNodeImplementation({
        generateOperation: (phase) => `${phase} % 1`
    }),
};
const builders = {
    'osc~': builder,
    'phasor~': builder,
};

export { builders, nodeImplementations };
