import { assertOptionalString } from '../validation.js';
import { messageBuses } from '../global-code/buses.js';
import { commonsWaitEngineConfigure } from '../../../../WebPd_compiler/dist/src/stdlib/commons.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
const stateVariables = {};
// ------------------------------- node builder ------------------------------ //
const builder = {
    translateArgs: (pdNode) => ({
        busName: assertOptionalString(pdNode.args[0]) || '',
    }),
    build: () => ({
        inlets: {},
        outlets: {
            '0': { type: 'message', id: '0' },
        },
        isPushingMessages: true
    }),
};
// ------------------------------- generateDeclarations ------------------------------ //
const generateDeclarations = ({ compilation: { codeVariableNames: { nodes } }, node: { id, args }, }) => `
    commons_waitEngineConfigure(() => {
        msgBusSubscribe("${args.busName}", ${nodes[id].snds.$0})
    })
`;
// ------------------------------------------------------------------- //
const nodeImplementation = {
    stateVariables,
    generateDeclarations,
    dependencies: [messageBuses, commonsWaitEngineConfigure],
};

export { builder, nodeImplementation };
