import { assertOptionalString } from '../validation.js';
import { messageBuses } from '../global-code/buses.js';
import { coldStringInlet } from '../standard-message-receivers.js';
import { commonsWaitEngineConfigure } from '../../../../WebPd_compiler/dist/src/stdlib/commons.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
const stateVariables = {
    busName: 1,
};
// ------------------------------- node builder ------------------------------ //
const builder = {
    translateArgs: (pdNode) => ({
        busName: assertOptionalString(pdNode.args[0]) || '',
    }),
    build: () => ({
        inlets: {
            '0': { type: 'message', id: '0' },
            '1': { type: 'message', id: '1' },
        },
        outlets: {},
    }),
};
// ------------------------------- generateDeclarations ------------------------------ //
const generateDeclarations = ({ state, macros: { Var }, node: { args }, }) => `
    let ${Var(state.busName, 'string')} = "${args.busName}"
`;
// ------------------------------- generateMessageReceivers ------------------------------ //
const generateMessageReceivers = ({ state, globs, }) => ({
    '0': `
    msgBusPublish(${state.busName}, ${globs.m})
    return
    `,
    '1': coldStringInlet(globs.m, state.busName)
});
// ------------------------------------------------------------------- //
const nodeImplementation = {
    stateVariables,
    generateDeclarations,
    generateMessageReceivers,
    dependencies: [messageBuses, commonsWaitEngineConfigure],
};

export { builder, nodeImplementation };
