import { assertOptionalString } from '../validation.js';
import { signalBuses } from '../global-code/buses.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
const stateVariables = {
    busName: 1,
};
// ------------------------------- node builder ------------------------------ //
const builder = {
    translateArgs: (pdNode) => ({
        busName: assertOptionalString(pdNode.args[0]) || '',
    }),
    build: () => ({
        inlets: {
            '0': { type: 'signal', id: '0' },
        },
        outlets: {},
        isPullingSignal: true,
    }),
};
// ------------------------------- generateDeclarations ------------------------------ //
const generateDeclarations = ({ state, node: { args }, macros: { Var } }) => `
    const ${Var(state.busName, 'string')} = "${args.busName}"
`;
// ------------------------------- generateLoop ------------------------------ //
const generateLoop = ({ ins, state }) => `
    setSignalBus(${state.busName}, ${ins.$0})
`;
// ------------------------------------------------------------------- //
const nodeImplementation = {
    generateLoop,
    stateVariables,
    generateDeclarations,
    dependencies: [signalBuses]
};

export { builder, nodeImplementation };
