import { bangUtils } from '../global-code/core.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
const stateVariables = {
    currentValue: 1,
};
// ------------------------------- node builder ------------------------------ //
const builder = {
    translateArgs: () => ({}),
    build: () => ({
        inlets: {
            '0': { type: 'signal', id: '0' },
            '0_message': { type: 'message', id: '0_message' },
        },
        outlets: {
            '0': { type: 'message', id: '0' },
        },
        isPullingSignal: true,
    }),
    configureMessageToSignalConnection: (inletId) => {
        if (inletId === '0') {
            return { reroutedMessageInletId: '0_message' };
        }
        return undefined;
    },
};
// ------------------------------- generateDeclarations ------------------------------ //
const generateDeclarations = ({ state, macros: { Var } }) => `
    let ${Var(state.currentValue, 'Float')} = 0
`;
// ------------------------------- generateLoop ------------------------------ //
const generateLoop = ({ ins, state }) => `
    ${state.currentValue} = ${ins.$0}
`;
// ------------------------------- generateMessageReceivers ------------------------------ //
const generateMessageReceivers = ({ state, globs, snds }) => ({
    '0_message': `
        if (msg_isBang(${globs.m})) {
            ${snds.$0}(msg_floats([${state.currentValue}]))
            return 
        }
    `,
});
// ------------------------------------------------------------------- //
const nodeImplementation = {
    generateLoop,
    stateVariables,
    generateMessageReceivers,
    generateDeclarations,
    dependencies: [bangUtils]
};

export { builder, nodeImplementation };
