import { ValidationError } from './validation.js';

const TYPE_ARGUMENTS = [
    'float',
    'bang',
    'symbol',
    'list',
    'anything',
];
const resolveTypeArgumentAlias = (value) => {
    switch (value) {
        case 'f':
            return 'float';
        case 'b':
            return 'bang';
        case 's':
            return 'symbol';
        case 'l':
            return 'list';
        case 'a':
            return 'anything';
        case 'p':
            return 'pointer';
        default:
            return value;
    }
};
const assertTypeArgument = (value) => {
    if (value === 'pointer') {
        throw new ValidationError(`"pointer" not supported (yet)`);
    }
    else if (!TYPE_ARGUMENTS.includes(value)) {
        throw new ValidationError(`invalid type ${value}`);
    }
    return value;
};
const renderMessageTransfer = (typeArgument, msgVariableName, index) => {
    switch (typeArgument) {
        case 'float':
            return `msg_floats([messageTokenToFloat(${msgVariableName}, ${index})])`;
        case 'bang':
            return `msg_bang()`;
        case 'symbol':
            return `msg_strings([messageTokenToString(${msgVariableName}, ${index})])`;
        case 'list':
        case 'anything':
            return `${msgVariableName}`;
        default:
            throw new Error(`type argument ${typeArgument} not supported (yet)`);
    }
};
const messageTokenToFloat = ({ macros: { Func, Var } }) => `
    function messageTokenToFloat ${Func([
    Var('m', 'Message'),
    Var('i', 'Int')
], 'Float')} {
        if (msg_isFloatToken(m, i)) {
            return msg_readFloatToken(m, i)
        } else {
            return 0
        }
    }
`;
const messageTokenToString = ({ macros: { Func, Var } }) => `
    function messageTokenToString ${Func([
    Var('m', 'Message'),
    Var('i', 'Int')
], 'string')} {
        if (msg_isStringToken(m, i)) {
            const ${Var('str', 'string')} = msg_readStringToken(m, i)
            if (str === 'bang') {
                return 'symbol'
            } else {
                return str
            }
        } else {
            return 'float'
        }
    }
`;

export { assertTypeArgument, messageTokenToFloat, messageTokenToString, renderMessageTransfer, resolveTypeArgumentAlias };
