import { Code } from '@webpd/compiler';
import { NodeImplementations, NodeVariableNames } from '@webpd/compiler/src/compile/types';
import { NodeBuilder } from '../../compile-dsp-graph/types';
interface NodeArguments {
    tokenizedExpressions: Array<Array<ExpressionToken>>;
}
declare const stateVariables: {
    floatInputs: number;
    stringInputs: number;
    outputs: number;
};
export declare const TOKENIZE_REGEXP: RegExp;
interface ExpressionTokenFloat {
    type: 'float';
    id: number;
}
interface ExpressionTokenSignal {
    type: 'signal';
    id: number;
}
interface ExpressionTokenInt {
    type: 'int';
    id: number;
}
interface ExpressionTokenString {
    type: 'string';
    id: number;
}
interface ExpressionTokenIndexingStart {
    type: 'indexing-start';
}
interface ExpressionTokenIndexingEnd {
    type: 'indexing-end';
}
interface ExpressionTokenRaw {
    type: 'raw';
    content: string;
}
type InputToken = ExpressionTokenString | ExpressionTokenFloat | ExpressionTokenInt | ExpressionTokenSignal;
export type ExpressionToken = ExpressionTokenRaw | ExpressionTokenIndexingStart | ExpressionTokenIndexingEnd | InputToken;
export declare const tokenizeExpression: (expression: string) => ExpressionToken[];
export declare const renderTokenizedExpression: (state: {
    floatInputs: string;
    stringInputs: string;
    outputs: string;
}, ins: NodeVariableNames['ins'] | null, tokens: Array<ExpressionToken>) => Code;
export declare const listInputs: (tokenizedExpressions: Array<Array<ExpressionToken>>) => InputToken[];
declare const nodeImplementations: NodeImplementations;
declare const builders: {
    expr: NodeBuilder<NodeArguments>;
    'expr~': NodeBuilder<NodeArguments>;
};
export { builders, nodeImplementations, NodeArguments };
